import React, { useEffect, useState } from 'react';
import { StatusBar } from 'react-native';
import { RFValue } from 'react-native-responsive-fontsize';
import { useNetInfo } from '@react-native-community/netinfo';
import { useNavigation } from '@react-navigation/native';
import { synchronize } from '@nozbe/watermelondb/sync';
import { Q } from '@nozbe/watermelondb';
import { database } from '../../database';

import Logo from '../../assets/logo.svg';
import { api } from '../../services/api';

import { Product } from '../../components/Product';
import { Search } from '../../components/Search';
import { Produto as ModelProduto } from '../../database/model/Produto';

import { LoadAnimation } from '../../components/LoadAnimation';
import base64 from 'react-native-base64';

import {
    Container,
    Header,
    TotalCars,
    CarList,
    HeaderContent,
    Connected,
    ConnectedText
} from './styles';

export function Home() {
    const [produtos, setProdutos] = useState<ModelProduto[]>([]);
    const [loading, setLoading] = useState(false);
    const [search, setSearch] = useState('');
    const NetInfo = useNetInfo();
    const navigation = useNavigation<any>();

    /*function handleCarDetails(car: ModelCar) {
        navigation.navigate('CarDetails', { car })
    }*/

    async function offlineSynchronize() {
        await synchronize({
            database,
            pullChanges: async ({ lastPulledAt }) => {
                const authHeader = 'Basic ' + base64.encode('gefw:gefw2020');
                api.defaults.headers.common['Authorization'] = authHeader;
                const response = await api
                    .get(`products/sync/pull?lastPulledVersion=${0 || 0}`);
                console.log(response.data);
                const { changes, latestVersion } = response.data;
                return { changes, timestamp: latestVersion }
            },
            pushChanges: async ({ changes }) => {
                const user = changes.users;
                await api.post('/users/sync', user);
            },
        });
    }

    async function fetchProdutos(value: string) {
        let isMounted = true;
        try {
            const produtoCollection = database.get<ModelProduto>('produtos');
            const produtos = await produtoCollection.query(
                Q.where("nome", Q.like(`%${Q.sanitizeLikeString(value)}%`))
            ).fetch();


            if (isMounted) {
                setProdutos(produtos);
            }
        } catch (error) {
            console.log(error);
        } finally {
            if (isMounted) {
                setLoading(false);
            }
        }
        return () => {
            isMounted = false;
        };
    };

    useEffect(() => {
        fetchProdutos(search);
    }, []);

    function handleSearch() {
        fetchProdutos(search);
    }

    function handleSearchClear() {
        setSearch('');
        fetchProdutos('');
    }

    useEffect(() => {
        if (NetInfo.isConnected === true) {
            offlineSynchronize();
        }
    }, [NetInfo.isConnected]);

    return (
        <Container>
            <StatusBar
                barStyle="light-content"
                backgroundColor="transparent"
                translucent
            />
            <Header>
                <HeaderContent>
                    <Logo
                        width={RFValue(108)}
                        height={RFValue(12)}
                    />
                    {
                        !loading &&
                        <TotalCars>
                            {produtos.length} produto(s) encontrado(s)
                        </TotalCars>
                    }

                </HeaderContent>
            </Header>

            <Search
                onChangeText={setSearch}
                value={search}
                onSearch={handleSearch}
                onClear={handleSearchClear}
            />

            {!NetInfo.isConnected &&
                <Connected>
                    <ConnectedText>
                        Você está off-Line!!
                    </ConnectedText>
                </Connected>
            }
            {loading ? <LoadAnimation /> :
                <CarList
                    data={produtos}
                    keyExtractor={item => item.id}
                    renderItem={({ item }) =>
                        <Product data={item} onPress={() => { }} />}
                />
            }
        </Container >
    );
}